@@ -22,6 +22,7 @@
 #include <config.h>
 #include <libgnome/libgnome.h>
 #include <sys/types.h>
+#include <sys/socket.h>
 #include <unistd.h>
 #include <sys/stat.h>
 #include <fcntl.h>
@@ -37,6 +38,14 @@
 #include "filecheck.h"
 #include "auth.h"
 
+/* Ensure we know about FamilyInternetV6 even if what we're compiling
+   against doesn't */
+#ifdef ENABLE_IPV6
+#ifndef FamilyInternetV6
+#define FamilyInternetV6	6
+#endif /* ! FamilyInternetV6 */
+#endif /* ENABLE_IPV6 */
+
 /* Local prototypes */
 static FILE *gdm_auth_purge (GdmDisplay *d, FILE *af, gboolean remove_when_empty);
 
@@ -402,7 +411,7 @@ get_local_auths (GdmDisplay *d)
 	    /* we should write out an entry for d->addr since
 	       possibly it is not in d->addrs */
 #ifdef ENABLE_IPV6
-	    if ( ! add_auth_entry (d, &auths, NULL, NULL, FamilyInternet, (char *)((d->addr6).s6_addr), sizeof (struct in6_addr)))
+	    if ( ! add_auth_entry (d, &auths, NULL, NULL, FamilyInternetV6, (char *)((d->addr6).s6_addr), sizeof (struct in6_addr)))
 		    goto get_local_auth_error;
 	    else
 #endif
@@ -431,7 +440,7 @@ get_local_auths (GdmDisplay *d)
 		    if (memcmp (ia6, &(d->addr6), sizeof (struct in6_addr)) == 0)
 			    continue;
 
-		    if ( ! add_auth_entry (d, &auths, NULL, NULL, FamilyInternet, (char *)(ia6->s6_addr), sizeof (struct in6_addr)))
+		    if ( ! add_auth_entry (d, &auths, NULL, NULL, FamilyInternetV6, (char *)(ia6->s6_addr), sizeof (struct in6_addr)))
 			    goto get_local_auth_error;
 
 		    if (gdm_is_loopback_addr6 (ia6))
@@ -462,7 +471,7 @@ get_local_auths (GdmDisplay *d)
 
 #ifdef ENABLE_IPV6
 	    if (ia->sa_family == AF_INET6) {
-		    if ( ! add_auth_entry (d, &auths, NULL, NULL, FamilyInternet, (char *)(((struct sockaddr_in6 *)ia)->sin6_addr.s6_addr), sizeof (struct in6_addr)))
+		    if ( ! add_auth_entry (d, &auths, NULL, NULL, FamilyInternetV6, (char *)(((struct sockaddr_in6 *)ia)->sin6_addr.s6_addr), sizeof (struct in6_addr)))
 			    goto get_local_auth_error;
 
 		    if (gdm_is_loopback_addr6 (&((struct sockaddr_in6 *)ia)->sin6_addr))
@@ -485,7 +494,7 @@ get_local_auths (GdmDisplay *d)
 
 #ifdef ENABLE_IPV6
 	    if (d->addrtype == AF_INET6) {
-		    if ( ! add_auth_entry (d, &auths, NULL, NULL, FamilyInternet, lo6, sizeof (struct in6_addr)))
+		    if ( ! add_auth_entry (d, &auths, NULL, NULL, FamilyInternetV6, lo6, sizeof (struct in6_addr)))
 			    goto get_local_auth_error;
 	    }
 	    else
